library(AER)
data(HMDA)
head(HMDA)
#Home Mortgage Disclosure Act Data
summary(HMDA)
str(HMDA)
#deny: was the mortgage denied? (No,Yes)
HMDA$deny
#pirat: Payments to income ratio.
HMDA$pirat

# The variable we are interested in modelling is deny, an indicator for whether an applicant’s
# mortgage application has been accepted (deny = no) or denied (deny = yes).
# We use as regressor pirat (Payments to income ratio: it shows the percentage of the subject
# monthly income taken up by a debt payment)

# We estimate this model just as any other linear regression model using lm(). 
linearmodel<-lm(deny~pirat, data = HMDA)
# lm() does not accept the dependent variable to be of class factor: the variable "deny" 
# must be converted to a numeric variable using as.numeric().
HMDA$deny<-as.numeric(HMDA$deny)-1
linearmodel<-lm(deny~pirat, data = HMDA)
summary(linearmodel)

# plot the data
plot(x = HMDA$pirat, 
     y = HMDA$deny,
     main = "Scatterplot Mortgage Application Denial and the Payment-to-Income Ratio",
     xlab = "P/I ratio",
     ylab = "Deny",
     pch = 20,
     ylim = c(-0.4, 1.4),
     cex.main = 0.8)

# add horizontal dashed lines and text
abline(h = 1, lty = 2, col = "darkred")
abline(h = 0, lty = 2, col = "darkred")
text(2.5, 0.9, cex = 0.8, "Mortgage denied")
text(2.5, -0.1, cex= 0.8, "Mortgage approved")

# add the estimated regression line
abline(linearmodel, 
       lwd = 1.8, 
       col = "steelblue")


coeftest(linearmodel, vcov. = vcovHC, type = "HC1")


#According to the estimated model, a payment-to-income ratio of 1  is associated with an 
#expected probability of mortgage application denial of roughly  50% . The model indicates 
#that there is a positive relation between the payment-to-income ratio and the probability 
#of a denied mortgage application so individuals with a high ratio of loan payments to income
#are more likely to be rejected

# Let us consider an additional regressor:  "afam"  which equals  1 if the applicant is 
# an African American  and equals  0  otherwise. Such a specification is the baseline 
#for investigating if there is racial discrimination in the mortgage market: if being black 
# has a significant (positive) influence on the probability of a loan denial when we control
# for factors that allow for an objective assessment of an applicants credit worthiness,
# this is an indicator for discrimination.

HMDA$afam<-as.numeric(HMDA$afam)-1
# estimate the model
linearmodel2 <- lm(deny ~ pirat + afam, data = HMDA)
coeftest(linearmodel2, vcov. = vcovHC)

# The coefficient on  "afam"  is positive and significantly different from zero at the 0.01% 
# level. The interpretation is that, holding constant the  P/I ratio , being black increases 
# the probability of a mortgage application denial by about  17.7%. This finding is 
# compatible with racial discrimination. However, it might be distorted by omitted variable 
# bias so discrimination could be a premature conclusion.


###########################################################################################
###########################################################################################
###########################################################################################


#We now estimate a simple Probit model of the probability of a mortgage denial.

denyprobit <- glm(deny ~ pirat, 
                  family = binomial(link = "probit"), 
                  data = HMDA)
summary(denyprobit)
coeftest(denyprobit, vcov. = vcovHC, type = "HC1")
#Calculate the residuals
pearson.res<-residuals(denyprobit,type = "pearson") ## pearson residuals
dev.res<-residuals(denyprobit,type = "deviance") ## deviance residuals
#McFadden’s R-squared ranges from 0 to just under 1, with higher values indicating 
#a better model fit.
McFaddenR2<-1-denyprobit$deviance/denyprobit$null.deviance


# Just as in the linear probability model we find that the relation between the probability 
# of denial and the payments-to-income ratio is positive and that the corresponding 
# coefficient is highly significant.


# plot data
plot(x = HMDA$pirat, 
     y = HMDA$deny,
     main = "Probit Model of the Probability of Denial, Given P/I Ratio",
     xlab = "P/I ratio",
     ylab = "Deny",
     pch = 20,
     ylim = c(-0.4, 1.4),
     cex.main = 0.85)

# add horizontal dashed lines and text
abline(h = 1, lty = 2, col = "darkred")
abline(h = 0, lty = 2, col = "darkred")
text(2.5, 0.9, cex = 0.8, "Mortgage denied")
text(2.5, -0.1, cex= 0.8, "Mortgage approved")

# add estimated regression line
x <- seq(0, 3, 0.01)
y <- predict(denyprobit, list(pirat = x), type = "response")

lines(x, y, lwd = 1.5, col = "steelblue")

# The estimated regression function has a stretched “S”-shape which is typical for 
# the CDF of a continuous random variable with symmetric PDF like that of a normal random 
# variable. 

# We use predict() to compute the predicted change in the denial probability when  P/I ratio
# is increased from  0.3  to  0.4 .

# 1. compute predictions for P/I ratio = 0.3, 0.4
predictions <- predict(denyprobit, 
                       newdata = data.frame("pirat" = c(0.3, 0.4)),
                       type = "response")

# 2. Compute difference in probabilities
diff(predictions)

# We find that an increase in the payment-to-income ratio from  0.3  to  0.4  is predicted 
# to increase the probability of denial by approximately  6.2%.

predictionsNew <- predict(denyprobit, 
                          newdata = data.frame("pirat" = seq(from=0,to=1,by=0.1)),
                          type = "response")
diff(predictionsNew)


####################################################################

denyprobit2 <- glm(deny ~ pirat + afam, 
                   family = binomial(link = "probit"), 
                   data = HMDA)
coeftest(denyprobit2, vcov. = vcovHC, type = "HC1")
#Calculate the residuals
pearson.res<-residuals(denyprobit2,type = "pearson") ## pearson residuals
dev.res<-residuals(denyprobit2,type = "deviance") ## deviance residuals
#McFadden’s R-squared ranges from 0 to just under 1, with higher values indicating 
#a better model fit.
McFaddenR2<-1-denyprobit2$deviance/denyprobit2$null.deviance

# The coefficients are difficult to interpret but they indicate that, first, 
# African Americans have a higher probability of denial than white applicants, 
# holding constant the payments-to-income ratio and second, applicants with a 
# high payments-to-income ratio face a higher risk of being rejected.

#How big is the estimated difference in denial probabilities between two hypothetical 
#applicants with the same payments-to-income ratio? 
#As before, we may use predict()

# 1. compute predictions for P/I ratio = 0.3
predictions <- predict(denyprobit2, 
                       newdata = data.frame("afam" = c(0, 1), 
                                            "pirat" = c(0.3, 0.3)),
                       type = "response")

# 2. compute difference in probabilities
diff(predictions)

#The estimated difference in denial probabilities is about  15.8% .

###########################################################################################
###########################################################################################
###########################################################################################

# Let us consider a Logit model

denylogit <- glm(deny ~ pirat, 
                 family = binomial(link = "logit"), 
                 data = HMDA)

summary(denylogit)

#Calculate the residuals
pearson.res<-residuals(denylogit,type = "pearson") ## pearson residuals
dev.res<-residuals(denylogit,type = "deviance") ## deviance residuals
#McFadden’s R-squared ranges from 0 to just under 1, with higher values indicating 
#a better model fit.
McFaddenR2<-1-denylogit$deviance/denylogit$null.deviance


# plot data
plot(x = HMDA$pirat, 
     y = HMDA$deny,
     main = "Probit and Logit Models Model of the Probability of Denial, Given P/I Ratio",
     xlab = "P/I ratio",
     ylab = "Deny",
     pch = 20,
     ylim = c(-0.4, 1.4),
     cex.main = 0.9)

# add horizontal dashed lines and text
abline(h = 1, lty = 2, col = "darkred")
abline(h = 0, lty = 2, col = "darkred")
text(2, 0.9, cex = 0.8, "Mortgage denied")
text(2, -0.1, cex= 0.8, "Mortgage approved")

# add estimated regression line of Probit and Logit models
x <- seq(0, 3, 0.01)
y_probit <- predict(denyprobit, list(pirat = x), type = "response")
y_logit <- predict(denylogit, list(pirat = x), type = "response")

lines(x, y_probit, lwd = 1.5, col = "steelblue")
lines(x, y_logit, lwd = 1.5, col = "black", lty = 2)

# add a legend
legend("topright",
       #horiz = TRUE,
       legend = c("Probit", "Logit"),
       col = c("steelblue", "black"), 
       lty = c(1, 2))

# estimate a Logit regression with multiple regressors
denylogit2 <- glm(deny ~ pirat + afam, 
                  family = binomial(link = "logit"), 
                  data = HMDA)

summary(denylogit2)
#Calculate the residuals
pearson.res<-residuals(denylogit2,type = "pearson") ## pearson residuals
dev.res<-residuals(denylogit2,type = "deviance") ## deviance residuals
#McFadden’s R-squared ranges from 0 to just under 1, with higher values indicating 
#a better model fit.
McFaddenR2<-1-denylogit2$deviance/denylogit2$null.deviance


# As for the Probit model all model coefficients are highly significant 
# and we obtain positive estimates for the coefficients on  P/I ratio  and  afam. 
# For comparison we compute the predicted probability of denial for two hypothetical 
# applicants that differ in race and have a  P/I ratio  of  0.3.

predictions <- predict(denylogit2, 
                       newdata = data.frame("afam" = c(0, 1), 
                                            "pirat" = c(0.3, 0.3)),
                       type = "response")

diff(predictions)

#We find that the white applicant faces a denial probability of only  7.5% , 
# while the African American is rejected with a probability of  22.4% , 
# a difference of  14.9% .


###########################################################################################
###########################################################################################
###########################################################################################


#Comparison of the ModelsThe Probit model and the Logit model deliver only approximations 
#to the unknown population regression function  E(Y|X) . It is not obvious how to decide 
#which model to use in practice. 

#1) The linear probability model has the clear drawback of not being able to capture 
# the nonlinear nature of the population regression function and it may predict 
# probabilities to lie outside the interval [0,1].

# 2) Probit and Logit models are harder to interpret but capture the nonlinearities:
#both models produce predictions of probabilities that lie inside the interval [0,1].
